package dev.fyloz.musicplayer.core

import dev.fyloz.musicplayer.core.data.RepositoryInjection
import dev.fyloz.musicplayer.core.factory.TrackFactoryProxy
import dev.fyloz.musicplayer.core.http.configureTrackRoutes
import dev.fyloz.musicplayer.core.logic.TrackLogic
import dev.fyloz.musicplayer.modules.spotify.SpotifyInjection
import dev.fyloz.musicplayer.modules.spotify.SpotifyModule
import dev.fyloz.musicplayer.modules.spotify.SpotifyTrackFactory
import io.ktor.client.*
import io.ktor.client.engine.apache.*
import io.ktor.http.*
import io.ktor.serialization.kotlinx.json.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.engine.*
import io.ktor.server.netty.*
import io.ktor.server.plugins.callloging.*
import io.ktor.server.plugins.contentnegotiation.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.serialization.json.Json
import org.koin.dsl.module
import org.koin.ktor.ext.inject
import org.koin.ktor.plugin.Koin
import org.koin.logger.slf4jLogger
import org.slf4j.event.Level

fun main() {
    embeddedServer(Netty, port = 8080, host = "0.0.0.0", module = Application::module)
        .start(wait = true)
}

fun Application.module() {
    install(CallLogging) {
        level = Level.DEBUG
//        filter { call -> call.request.path().startsWith("/") }
    }

    install(ContentNegotiation) {
        json()
    }

    module {
        install(Koin) {
            slf4jLogger()
            modules(
                RepositoryInjection.koinBeans,
                SpotifyInjection.koinBeans,

                module {
                    single { TrackFactoryProxy() }
                    single { TrackLogic() }
                }
            )
        }
    }

    val spotifyModule = SpotifyModule()
    spotifyModule.configure(this)

    install(Routing) {
        spotifyModule.configureRoutes(this)

        route("/api/v1") {
            configureTrackRoutes()
        }
    }
}
