package dev.fyloz.musicplayer.core.logic

import dev.fyloz.musicplayer.core.data.TrackRepository
import dev.fyloz.musicplayer.core.factory.TrackFactoryProxy
import dev.fyloz.musicplayer.core.model.Track
import dev.fyloz.musicplayer.modules.spotify.SpotifyTrackFactory
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject
import java.util.UUID

class TrackLogic : KoinComponent {
    private val repository by inject<TrackRepository>()
    private val trackFactory by inject<TrackFactoryProxy>()

    fun getAll() = repository.findAll()
    fun getById(id: String) = repository.findById(id)

    suspend fun save(type: String, trackId: String): Track {
        val id = generateId()
        val track = trackFactory.createTrack(type, id, trackId)
        repository.save(track)
        return track
    }

    private fun generateId() = UUID.randomUUID().toString()
}
