package dev.fyloz.musicplayer.modules

import dev.fyloz.musicplayer.core.factory.TrackFactory
import dev.fyloz.musicplayer.core.factory.TrackFactoryProxy
import io.ktor.server.application.*
import io.ktor.server.routing.*
import org.koin.ktor.ext.inject

abstract class Module(private val moduleName: String) {
    open fun configure(app: Application) {
        with(app) {
            val trackFactoryProxy by inject<TrackFactoryProxy>()
            trackFactoryProxy.registerFactory(moduleName, getTrackFactory())
        }
    }

    fun configureRoutes(routing: Routing) {
        with(routing) {
            route("/api/v1/$moduleName") {
                configureApiRoutes()
            }

            route("/module/$moduleName") {
                configureModuleRoutes()
            }
        }
    }

    protected open fun Route.configureApiRoutes() {
    }

    protected open fun Route.configureModuleRoutes() {
    }

    protected abstract fun getTrackFactory(): TrackFactory
}
