package dev.fyloz.musicplayer.modules.spotify

import dev.fyloz.musicplayer.modules.Module
import io.ktor.client.*
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.response.*
import io.ktor.server.routing.*

class SpotifyModule : Module("spotify") {
    override fun configure(app: Application) {
        super.configure(app)
        with(app) {
            authentication {
                oauth("auth-oauth-spotify") {
                    urlProvider = { "http://localhost:8080/module/spotify/login-callback" }
                    providerLookup = {
                        OAuthServerSettings.OAuth2ServerSettings(
                            name = "spotify",
                            authorizeUrl = "https://accounts.spotify.com/authorize",
                            accessTokenUrl = "https://accounts.spotify.com/api/token",
                            requestMethod = HttpMethod.Post,
                            clientId = "1372bd3ebcad4f889994f9a3f675472b",
                            clientSecret = "26ac249dc5ca4a309aa08f8cfcec9a60"
                        )
                    }
                    client = HttpClient()
                }
            }
        }
    }

    override fun Route.configureModuleRoutes() {
        authenticate("auth-oauth-spotify") {
            get("/login") {
                call.respondRedirect("https://accounts.spotify.com/authorize")
            }

            get("/login-callback") {
                val principal: OAuthAccessTokenResponse.OAuth2? = call.principal()
                if (principal != null) {
                    call.response.cookies.append("Spotify-Access-Token", principal.accessToken)

                    if (principal.refreshToken != null) {
                        call.response.cookies.append("Spotify-Refresh-Token", principal.refreshToken!!)
                    }

//                    SpotifyTokenLogic(http)
                } else {
                    call.respond(HttpStatusCode.Unauthorized)
                }
            }
        }
    }

    override fun getTrackFactory() = SpotifyTrackFactory()
}
